@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000 Ximian Inc.
+ *  Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
  *
  *  Authors: Michael Zucchi <notzed@ximian.com>
  *
@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #include "camel-mime-filter-index.h"
@@ -30,7 +30,7 @@ CamelType
 camel_mime_filter_index_get_type (void)
 {
 	static CamelType type = CAMEL_INVALID_TYPE;
-	
+
 	if (type == CAMEL_INVALID_TYPE) {
 		type = camel_type_register (camel_mime_filter_get_type (), "CamelMimeFilterIndex",
 					    sizeof (CamelMimeFilterIndex),
@@ -40,7 +40,7 @@ camel_mime_filter_index_get_type (void)
 					    NULL,
 					    (CamelObjectFinalizeFunc) camel_mime_filter_index_finalize);
 	}
-	
+
 	return type;
 }
 
@@ -55,7 +55,7 @@ camel_mime_filter_index_finalize(CamelObject *o)
 }
 
 static void
-complete(CamelMimeFilter *mf, char *in, size_t len, size_t prespace, char **out, size_t *outlenptr, size_t *outprespace)
+complete(CamelMimeFilter *mf, const gchar *in, gsize len, gsize prespace, gchar **out, gsize *outlenptr, gsize *outprespace)
 {
 	CamelMimeFilterIndex *f = (CamelMimeFilterIndex *)mf;
 
@@ -67,13 +67,13 @@ complete(CamelMimeFilter *mf, char *in, size_t len, size_t prespace, char **out,
 	camel_index_name_add_buffer(f->name, NULL, 0);
 
 donothing:
-	*out = in;
+	*out = (gchar *) in;
 	*outlenptr = len;
 	*outprespace = prespace;
 }
 
 static void
-filter(CamelMimeFilter *mf, char *in, size_t len, size_t prespace, char **out, size_t *outlenptr, size_t *outprespace)
+filter(CamelMimeFilter *mf, const gchar *in, gsize len, gsize prespace, gchar **out, gsize *outlenptr, gsize *outprespace)
 {
 	CamelMimeFilterIndex *f = (CamelMimeFilterIndex *)mf;
 
@@ -84,7 +84,7 @@ filter(CamelMimeFilter *mf, char *in, size_t len, size_t prespace, char **out, s
 	camel_index_name_add_buffer(f->name, in, len);
 
 donothing:
-	*out = in;
+	*out = (gchar *) in;
 	*outlenptr = len;
 	*outprespace = prespace;
 }
@@ -93,7 +93,7 @@ static void
 camel_mime_filter_index_class_init (CamelMimeFilterIndexClass *klass)
 {
 	CamelMimeFilterClass *filter_class = (CamelMimeFilterClass *) klass;
-	
+
 	camel_mime_filter_index_parent = CAMEL_MIME_FILTER_CLASS (camel_type_get_global_classfuncs (camel_mime_filter_get_type ()));
 
 	/*filter_class->reset = reset;*/
@@ -105,8 +105,8 @@ camel_mime_filter_index_class_init (CamelMimeFilterIndexClass *klass)
  * camel_mime_filter_index_new:
  *
  * Create a new #CamelMimeFilterIndex object
- * 
- * Returns a new #CamelMimeFilterIndex object
+ *
+ * Returns: a new #CamelMimeFilterIndex object
  **/
 CamelMimeFilterIndex *
 camel_mime_filter_index_new (void)
@@ -115,14 +115,13 @@ camel_mime_filter_index_new (void)
 	return new;
 }
 
-
 /**
  * camel_mime_filter_index_new_index:
  * @index: a #CamelIndex object
  *
  * Create a new #CamelMimeFilterIndex based on @index.
  *
- * Returns a new #CamelMimeFilterIndex object
+ * Returns: a new #CamelMimeFilterIndex object
  **/
 CamelMimeFilterIndex *
 camel_mime_filter_index_new_index (CamelIndex *index)
@@ -139,7 +138,6 @@ camel_mime_filter_index_new_index (CamelIndex *index)
 
 /* Set the match name for any indexed words */
 
-
 /**
  * camel_mime_filter_index_set_name:
  * @filter: a #CamelMimeFilterIndex object
@@ -157,7 +155,6 @@ camel_mime_filter_index_set_name (CamelMimeFilterIndex *filter, CamelIndexName *
 		camel_object_ref (name);
 }
 
-
 /**
  * camel_mime_filter_index_set_index:
  * @filter: a #CamelMimeFilterIndex object
@@ -169,8 +166,8 @@ void
 camel_mime_filter_index_set_index (CamelMimeFilterIndex *filter, CamelIndex *index)
 {
 	if (filter->index) {
-		char *out;
-		size_t outlen, outspace;
+		gchar *out;
+		gsize outlen, outspace;
 
 		camel_mime_filter_complete((CamelMimeFilter *)filter, "", 0, 0, &out, &outlen, &outspace);
 		camel_object_unref (index);
