@@ -5,8 +5,8 @@
  *
  * Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
  *
- * This program is free software; you can redistribute it and/or 
- * modify it under the terms of version 2 of the GNU Lesser General Public 
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of version 2 of the GNU Lesser General Public
  * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -31,7 +31,6 @@
 #endif
 
 #include <pthread.h>
-#include <libedataserver/e-msgport.h>
 
 G_BEGIN_DECLS
 
@@ -39,7 +38,7 @@ struct _CamelFolderPrivate {
 	GStaticRecMutex lock;
 	GStaticMutex change_lock;
 	/* must require the 'change_lock' to access this */
-	int frozen;
+	gint frozen;
 	struct _CamelFolderChangeInfo *changed_frozen; /* queues changed events */
 };
 
@@ -52,7 +51,6 @@ struct _CamelFolderPrivate {
 #define CAMEL_FOLDER_REC_UNLOCK(f, l) \
 	(g_static_rec_mutex_unlock(&((CamelFolder *) (f))->priv->l))
 
-
 struct _CamelStorePrivate {
 	GStaticRecMutex folder_lock;	/* for locking folder operations */
 };
@@ -62,7 +60,6 @@ struct _CamelStorePrivate {
 #define CAMEL_STORE_UNLOCK(f, l) \
 	(g_static_rec_mutex_unlock(&((CamelStore *) (f))->priv->l))
 
-
 struct _CamelTransportPrivate {
 	GMutex *send_lock;   /* for locking send operations */
 };
@@ -72,7 +69,6 @@ struct _CamelTransportPrivate {
 #define CAMEL_TRANSPORT_UNLOCK(f, l) \
 	(g_mutex_unlock(((CamelTransport *) (f))->priv->l))
 
-
 struct _CamelServicePrivate {
 	GStaticRecMutex connect_lock;	/* for locking connection operations */
 	GStaticMutex connect_op_lock;	/* for locking the connection_op */
@@ -89,18 +85,17 @@ struct _CamelServicePrivate {
 #define CAMEL_SERVICE_REC_TRYLOCK(f, l) \
 	(g_static_rec_mutex_trylock(&((CamelService *) (f))->priv->l))
 
-
 struct _CamelSessionPrivate {
 	GMutex *lock;		/* for locking everything basically */
 	GMutex *thread_lock;	/* locking threads */
 
-	int thread_id;
+	gint thread_id;
 	GHashTable *thread_active;
 	GThreadPool *thread_pool;
 
 	GHashTable *thread_msg_op;
 	GHashTable *junk_headers;
-	
+
 };
 
 #define CAMEL_SESSION_LOCK(f, l) \
@@ -108,7 +103,6 @@ struct _CamelSessionPrivate {
 #define CAMEL_SESSION_UNLOCK(f, l) \
 	(g_mutex_unlock(((CamelSession *) (f))->priv->l))
 
-
 /* most of this stuff really is private, but the lock can be used by subordinate classes */
 struct _CamelFolderSummaryPrivate {
 	GHashTable *filter_charset;	/* CamelMimeFilterCharset's indexed by source charset */
@@ -123,12 +117,16 @@ struct _CamelFolderSummaryPrivate {
 	struct _CamelStreamFilter *filter_stream;
 
 	struct _CamelIndex *index;
-	
+
 	GMutex *summary_lock;	/* for the summary hashtable/array */
 	GMutex *io_lock;	/* load/save lock, for access to saved_count, etc */
 	GMutex *filter_lock;	/* for accessing any of the filtering/indexing stuff, since we share them */
 	GMutex *alloc_lock;	/* for setting up and using allocators */
 	GMutex *ref_lock;	/* for reffing/unreffing messageinfo's ALWAYS obtain before summary_lock */
+	GHashTable *flag_cache;
+
+	gboolean need_preview;
+	GHashTable *preview_updates;
 };
 
 #define CAMEL_SUMMARY_LOCK(f, l) \
@@ -136,7 +134,6 @@ struct _CamelFolderSummaryPrivate {
 #define CAMEL_SUMMARY_UNLOCK(f, l) \
 	(g_mutex_unlock(((CamelFolderSummary *) (f))->priv->l))
 
-
 struct _CamelStoreSummaryPrivate {
 	GMutex *summary_lock;	/* for the summary hashtable/array */
 	GMutex *io_lock;	/* load/save lock, for access to saved_count, etc */
@@ -149,15 +146,15 @@ struct _CamelStoreSummaryPrivate {
 #define CAMEL_STORE_SUMMARY_UNLOCK(f, l) \
 	(g_mutex_unlock(((CamelStoreSummary *) (f))->priv->l))
 
-
 struct _CamelVeeFolderPrivate {
 	gboolean destroyed;
 	GList *folders;			/* lock using subfolder_lock before changing/accessing */
 	GList *folders_changed;		/* for list of folders that have changed between updates */
-	
+
 	GMutex *summary_lock;		/* for locking vfolder summary */
 	GMutex *subfolder_lock;		/* for locking the subfolder list */
 	GMutex *changed_lock;		/* for locking the folders-changed list */
+	gint unread_vfolder;
 };
 
 #define CAMEL_VEE_FOLDER_LOCK(f, l) \
@@ -165,7 +162,6 @@ struct _CamelVeeFolderPrivate {
 #define CAMEL_VEE_FOLDER_UNLOCK(f, l) \
 	(g_mutex_unlock(((CamelVeeFolder *) (f))->priv->l))
 
-
 struct _CamelDataWrapperPrivate {
 	pthread_mutex_t stream_lock;
 };
@@ -175,7 +171,6 @@ struct _CamelDataWrapperPrivate {
 #define CAMEL_DATA_WRAPPER_UNLOCK(dw, l) \
 	(pthread_mutex_unlock(&((CamelDataWrapper *) (dw))->priv->l))
 
-
 /* most of this stuff really is private, but the lock can be used by subordinate classes */
 struct _CamelCertDBPrivate {
 	GMutex *db_lock;	/* for the db hashtable/array */
@@ -192,9 +187,9 @@ struct _CamelCertDBPrivate {
 #ifdef G_OS_WIN32
 #define fsync(fd) _commit(fd)
 
-const char *_camel_get_localedir (void) G_GNUC_CONST;
-const char *_camel_get_libexecdir (void) G_GNUC_CONST;
-const char *_camel_get_providerdir (void) G_GNUC_CONST;
+const gchar *_camel_get_localedir (void) G_GNUC_CONST;
+const gchar *_camel_get_libexecdir (void) G_GNUC_CONST;
+const gchar *_camel_get_providerdir (void) G_GNUC_CONST;
 
 #undef EVOLUTION_LOCALEDIR
 #define EVOLUTION_LOCALEDIR _camel_get_localedir ()
