@@ -1,13 +1,13 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /* camel-imap-store.h : class for an imap store */
 
-/* 
+/*
  * Authors: Jeffrey Stedfast <fejj@ximian.com>
  *
  * Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
  *
- * This program is free software; you can redistribute it and/or 
- * modify it under the terms of version 2 of the GNU Lesser General Public 
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of version 2 of the GNU Lesser General Public
  * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -21,23 +21,22 @@
  * USA
  */
 
-
 #ifndef CAMEL_IMAP_STORE_H
 #define CAMEL_IMAP_STORE_H 1
 
 #include "camel-imap-types.h"
-#include <camel/camel-disco-store.h>
+#include <camel/camel-msgport.h>
+#include <camel/camel-offline-store.h>
 #include <sys/time.h>
 
 #ifdef ENABLE_THREADS
-#include <libedataserver/e-msgport.h>
 
 G_BEGIN_DECLS
 
 typedef struct _CamelImapMsg CamelImapMsg;
 
 struct _CamelImapMsg {
-	EMsg msg;
+	CamelMsg msg;
 
 	void (*receive)(CamelImapStore *store, struct _CamelImapMsg *m);
 	void (*free)(CamelImapStore *store, struct _CamelImapMsg *m);
@@ -45,7 +44,7 @@ struct _CamelImapMsg {
 
 CamelImapMsg *camel_imap_msg_new(void (*receive)(CamelImapStore *store, struct _CamelImapMsg *m),
 				 void (*free)(CamelImapStore *store, struct _CamelImapMsg *m),
-				 size_t size);
+				 gsize size);
 void camel_imap_msg_queue(CamelImapStore *store, CamelImapMsg *msg);
 
 G_END_DECLS
@@ -60,13 +59,14 @@ G_END_DECLS
 G_BEGIN_DECLS
 
 enum {
-	CAMEL_IMAP_STORE_ARG_FIRST  = CAMEL_DISCO_STORE_ARG_FIRST + 100,
+	CAMEL_IMAP_STORE_ARG_FIRST  = CAMEL_OFFLINE_STORE_ARG_FIRST + 100,
 	CAMEL_IMAP_STORE_ARG_NAMESPACE,
 	CAMEL_IMAP_STORE_ARG_OVERRIDE_NAMESPACE,
 	CAMEL_IMAP_STORE_ARG_CHECK_ALL,
 	CAMEL_IMAP_STORE_ARG_FILTER_INBOX,
 	CAMEL_IMAP_STORE_ARG_FILTER_JUNK,
-	CAMEL_IMAP_STORE_ARG_FILTER_JUNK_INBOX
+	CAMEL_IMAP_STORE_ARG_FILTER_JUNK_INBOX,
+	CAMEL_IMAP_STORE_ARG_CHECK_LSUB
 };
 
 #define CAMEL_IMAP_STORE_NAMESPACE           (CAMEL_IMAP_STORE_ARG_NAMESPACE | CAMEL_ARG_STR)
@@ -75,6 +75,7 @@ enum {
 #define CAMEL_IMAP_STORE_FILTER_INBOX        (CAMEL_IMAP_STORE_ARG_FILTER_INBOX | CAMEL_ARG_INT)
 #define CAMEL_IMAP_STORE_FILTER_JUNK         (CAMEL_IMAP_STORE_ARG_FILTER_JUNK | CAMEL_ARG_BOO)
 #define CAMEL_IMAP_STORE_FILTER_JUNK_INBOX   (CAMEL_IMAP_STORE_ARG_FILTER_JUNK_INBOX | CAMEL_ARG_BOO)
+#define CAMEL_IMAP_STORE_CHECK_LSUB          (CAMEL_IMAP_STORE_ARG_CHECK_LSUB | CAMEL_ARG_BOO)
 
 /* CamelFolderInfo flags */
 #define CAMEL_IMAP_FOLDER_MARKED	     (1<<16)
@@ -106,19 +107,20 @@ typedef enum {
 #define IMAP_PARAM_FILTER_JUNK			(1 << 3)
 #define IMAP_PARAM_FILTER_JUNK_INBOX		(1 << 4)
 #define IMAP_PARAM_SUBSCRIPTIONS		(1 << 5)
+#define IMAP_PARAM_CHECK_LSUB			(1 << 6) /* check for new messages in subscribed folders */
 
 #define IMAP_FETCH_ALL_HEADERS 1
 #define IMAP_FETCH_MAILING_LIST_HEADERS 2 /* Fetches Minimal and Mailing List Headers. Default behavior */
 #define IMAP_FETCH_MINIMAL_HEADERS 3
 
 struct _CamelImapStore {
-	CamelDiscoStore parent_object;	
-	
+	CamelOfflineStore parent_object;
+
 	CamelStream *istream;
 	CamelStream *ostream;
 
 	struct _CamelImapStoreSummary *summary;
-	
+
 	/* Information about the command channel / connection status */
 	guint connected:1;
 	guint preauthed:1;
@@ -129,36 +131,33 @@ struct _CamelImapStore {
 	/* broken server - wont let us append with custom flags even if the folder allows them */
 	guint nocustomappend:1;
 
-	char tag_prefix;
+	gchar tag_prefix;
 	guint32 command;
 	CamelFolder *current_folder;
-	
+
 	/* Information about the server */
 	CamelImapServerLevel server_level;
 	guint32 capabilities, parameters;
-	/* NB: namespace should be handled by summary->namespace */
-	char *namespace, dir_sep, *base_url, *storage_path;
+	gchar *users_namespace, dir_sep, *base_url, *storage_path;
 	GHashTable *authtypes;
-	
+
 	time_t refresh_stamp;
 
 	guint32 headers;
-	char *custom_headers;
+	gchar *custom_headers;
 };
 
 typedef struct {
-	CamelDiscoStoreClass parent_class;
+	CamelOfflineStoreClass parent_class;
 
 } CamelImapStoreClass;
 
-
 /* Standard Camel function */
 CamelType camel_imap_store_get_type (void);
 
-
 gboolean camel_imap_store_connected (CamelImapStore *store, CamelException *ex);
 
-ssize_t camel_imap_store_readline (CamelImapStore *store, char **dest, CamelException *ex);
+gssize camel_imap_store_readline (CamelImapStore *store, gchar **dest, CamelException *ex);
 
 G_END_DECLS
 
