@@ -10,6 +10,7 @@
 
 #include "camel-imapx-folder.h"
 #include "camel-imapx-stream.h"
+#include "camel-imapx-store-summary.h"
 #include "camel-imapx-utils.h"
 #include "camel-imapx-exception.h"
 #include "libedataserver/e-memory.h"
@@ -35,6 +36,8 @@ imap_tokenise (register const gchar *str, register guint len)
 	return 0;
 }
 
+static void imapx_namespace_clear (CamelIMAPXStoreNamespace **ns);
+
 /* flag table */
 static struct {
 	gchar *name;
@@ -56,7 +59,8 @@ void
 imap_parse_flags(CamelIMAPXStream *stream, guint32 *flagsp, CamelFlag **user_flagsp, CamelException *ex)
 /* throws IO,PARSE exception */
 {
-	gint tok, len, i;
+	gint tok, i;
+	guint len;
 	guchar *token, *p, c;
 	guint32 flags = 0;
 
@@ -71,13 +75,13 @@ imap_parse_flags(CamelIMAPXStream *stream, guint32 *flagsp, CamelFlag **user_fla
 				// FIXME: ascii_toupper
 				while ((c=*p))
 					*p++ = toupper(c);
-				for (i=0;i<(gint)(sizeof(flag_table)/sizeof(flag_table[0]));i++)
-					if (!strcmp(token, flag_table[i].name)) {
+				for (i = 0; i < G_N_ELEMENTS (flag_table); i++)
+					if (!strcmp((gchar *)token, flag_table[i].name)) {
 						flags |= flag_table[i].flag;
 						goto found;
 					}
 				if (user_flagsp)
-					camel_flag_set(user_flagsp, token, TRUE);
+					camel_flag_set(user_flagsp, (gchar *)token, TRUE);
 			found:
 				tok = tok; /* fixes stupid warning */
 			} else if (tok != ')') {
@@ -103,7 +107,7 @@ imap_write_flags(CamelStream *stream, guint32 flags, CamelFlag *user_flags)
 	if (camel_stream_write(stream, "(", 1) == -1)
 		camel_exception_throw(1, "io error: %s", strerror(errno));
 
-	for (i=0;flags!=0 && i<(gint)(sizeof(flag_table)/sizeof(flag_table[0]));i++) {
+	for (i=0;flags!=0 && i< G_N_ELEMENTS (flag_table);i++) {
 		if (flag_table[i].flag & flags) {
 			if (camel_stream_write(stream, flag_table[i].name, strlen(flag_table[i].name)) == -1)
 				camel_exception_throw(1, "io error: %s", strerror(errno));
@@ -151,7 +155,8 @@ struct _capability_info *
 imap_parse_capability(CamelIMAPXStream *stream, CamelException *ex)
 {
 	gint tok, len, i;
-	guchar *token, *p, c;
+	guchar *token, *p, c, *temp;
+	gboolean free_token = FALSE;
 	struct _capability_info * volatile cinfo;
 
 	cinfo = g_malloc0(sizeof(*cinfo));
@@ -159,6 +164,9 @@ imap_parse_capability(CamelIMAPXStream *stream, CamelException *ex)
 	/* FIXME: handle auth types */
 	while (!camel_exception_is_set (ex) && (tok = camel_imapx_stream_token(stream, &token, &len, ex)) != '\n') {
 		switch (tok) {
+			case 43:
+				token = g_strconcat (token, "+", NULL);
+				free_token = TRUE;
 			case IMAP_TOK_TOKEN:
 			case IMAP_TOK_STRING:
 				p = token;
@@ -166,9 +174,14 @@ imap_parse_capability(CamelIMAPXStream *stream, CamelException *ex)
 					*p++ = toupper(c);
 			case IMAP_TOK_INT:
 				printf(" cap: '%s'\n", token);
-				for (i=0;i<(gint)(sizeof(capa_table)/sizeof(capa_table[0]));i++)
+				for (i = 0; i < G_N_ELEMENTS (capa_table); i++)
 					if (strcmp(token, capa_table[i].name))
 						cinfo->capa |= capa_table[i].flag;
+				if (free_token) {
+					g_free (token);
+					token = NULL;
+				}
+				free_token = FALSE;
 				break;
 			default:
 				camel_exception_set (ex, 1, "capability: expecting name");
@@ -189,6 +202,108 @@ void imap_free_capability(struct _capability_info *cinfo)
 	g_free(cinfo);
 }
 
+struct _CamelIMAPXNamespaceList *
+imap_parse_namespace_list (CamelIMAPXStream *stream, CamelException *ex)
+{
+	CamelIMAPXStoreNamespace *namespaces[3], *node, *tail;
+	CamelIMAPXNamespaceList *nsl = NULL;
+	gint tok, len, i;
+	guchar *token, *p, c;
+	gint n = 0;
+
+	nsl = g_malloc0(sizeof(CamelIMAPXNamespaceList));
+	nsl->personal = NULL;
+	nsl->shared = NULL;
+	nsl->other = NULL;
+
+	tok = camel_imapx_stream_token (stream, &token, &len, ex);
+	do {
+		namespaces[n] = NULL;
+		tail = (CamelIMAPXStoreNamespace *) &namespaces[n];
+
+		if (tok == '(') {
+			tok = camel_imapx_stream_token (stream, &token, &len, ex);
+
+			while (tok == '(') {
+				tok = camel_imapx_stream_token (stream, &token, &len, ex);
+				if (tok != IMAP_TOK_STRING) {
+					camel_exception_set (ex, 1, "namespace: expected a string path name");
+					goto exception;
+				}
+
+				node = g_new0 (CamelIMAPXStoreNamespace, 1);
+				node->next = NULL;
+				node->path = g_strdup (token);
+				g_message ("namespace: Node path is %s \n", node->path);
+
+				tok = camel_imapx_stream_token (stream, &token, &len, ex);
+
+				if (tok == IMAP_TOK_STRING) {
+					if (strlen (token) == 1) {
+						node->sep = *token;
+					} else {
+						if (*token)
+							node->sep = node->path [strlen (node->path) - 1];
+						else
+							node->sep = '\0';
+					}
+				} else if (tok == IMAP_TOK_TOKEN) {
+					/* will a NIL be possible here? */
+					node->sep = '\0';
+				} else {
+					camel_exception_set (ex, 1, "namespace: expected a string separator");
+					g_free (node->path);
+					g_free (node);
+					goto exception;
+				}
+
+				tail->next = node;
+				tail = node;
+
+				if (node->path [strlen (node->path) -1] == node->sep)
+					node->path [strlen (node->path) - 1] = '\0';
+
+				if (!g_ascii_strncasecmp (node->path, "INBOX", 5) &&
+						(node->path [6] == '\0' || node->path [6] == node->sep ))
+					memcpy (node->path, "INBOX", 5);
+				tok = camel_imapx_stream_token (stream, &token, &len, ex);
+				if (tok != ')') {
+					camel_exception_set (ex, 1, "namespace: expected a ')'");
+					goto exception;
+				}
+
+				tok = camel_imapx_stream_token (stream, &token, &len, ex);
+			}
+
+			if (tok != ')') {
+				camel_exception_set (ex, 1, "namespace: expected a ')'");
+				goto exception;
+			}
+
+		} else if (tok == IMAP_TOK_TOKEN && !strcmp (token, "NIL")) {
+			namespaces [n] = NULL;
+		} else {
+			camel_exception_set (ex, 1, "namespace: expected either a '(' or NIL");
+			goto exception;
+		}
+
+		tok = camel_imapx_stream_token (stream, &token, &len, ex);
+		n++;
+	} while (n < 3);
+
+	nsl->personal = namespaces [0];
+	nsl->shared = namespaces [1];
+	nsl->other = namespaces [2];
+
+	return nsl;
+exception:
+	g_free (nsl);
+	for (i=0; i < 3; i++)
+		imapx_namespace_clear (&namespaces [i]);
+
+	return NULL;
+}
+
 /*
 body            ::= "(" body_type_1part / body_type_mpart ")"
 
@@ -1250,7 +1365,7 @@ imap_parse_list(CamelIMAPXStream *is, CamelException *ex)
 			p = token;
 			while ((c=*p))
 				*p++ = toupper(c);
-			for (i=0;i<(gint)(sizeof(list_flag_table)/sizeof(list_flag_table[0]));i++)
+			for (i = 0; i < G_N_ELEMENTS (list_flag_table); i++)
 				if (!strcmp(token, list_flag_table[i].name))
 					linfo->flags |= list_flag_table[i].flag;
 		} else {
@@ -1447,3 +1562,64 @@ imapx_concat (CamelIMAPXStore *imap_store, const gchar *prefix, const gchar *suf
 	else
 		return g_strdup_printf ("%s%c%s", prefix, imap_store->dir_sep, suffix);
 }
+
+static void
+imapx_namespace_clear (CamelIMAPXStoreNamespace **ns)
+{
+	CamelIMAPXStoreNamespace *node, *next;
+
+	node = *ns;
+	while (node != NULL) {
+		next = node->next;
+		g_free (node->path);
+		g_free (node);
+		node = next;
+	}
+
+	*ns = NULL;
+}
+
+void
+camel_imapx_namespace_list_clear (struct _CamelIMAPXNamespaceList *nsl)
+{
+	imapx_namespace_clear (&nsl->personal);
+	imapx_namespace_clear (&nsl->shared);
+	imapx_namespace_clear (&nsl->other);
+
+	g_free (nsl);
+	nsl = NULL;
+}
+
+static CamelIMAPXStoreNamespace *
+imapx_namespace_copy (const CamelIMAPXStoreNamespace *ns)
+{
+	CamelIMAPXStoreNamespace *list, *node, *tail;
+
+	list = NULL;
+	tail = (CamelIMAPXStoreNamespace *) &list;
+
+	while (ns != NULL) {
+		tail->next = node = g_malloc (sizeof (CamelIMAPXStoreNamespace));
+		node->path = g_strdup (ns->path);
+		node->sep = ns->sep;
+		ns = ns->next;
+		tail = node;
+	}
+
+	tail->next = NULL;
+
+	return list;
+}
+
+struct _CamelIMAPXNamespaceList *
+camel_imapx_namespace_list_copy (const struct _CamelIMAPXNamespaceList *nsl)
+{
+	CamelIMAPXNamespaceList *new;
+
+	new = g_malloc (sizeof (CamelIMAPXNamespaceList));
+	new->personal = imapx_namespace_copy (nsl->personal);
+	new->other = imapx_namespace_copy (nsl->other);
+	new->shared = imapx_namespace_copy (nsl->shared);
+
+	return new;
+}
