@@ -85,6 +85,16 @@ update_preview (GtkFileChooser *chooser, gpointer user_data)
 	g_free (filename);
 }
 
+static void
+file_chooser_response (GtkDialog *dialog, gint response_id, GtkFileChooser *button)
+{
+	g_return_if_fail (button != NULL);
+
+	if (response_id == GTK_RESPONSE_NO) {
+		gtk_file_chooser_unselect_all (button);
+	}
+}
+
 static CategoryPropertiesDialog *
 load_properties_dialog (ECategoriesDialog *parent)
 {
@@ -92,6 +102,7 @@ load_properties_dialog (ECategoriesDialog *parent)
 	const gchar *ui_to_load[] = { "properties-dialog", NULL };
 	gchar *uifile;
 	GError *error = NULL;
+	GtkWidget *table;
 
 	prop_dialog = g_new0 (CategoryPropertiesDialog, 1);
 
@@ -122,21 +133,42 @@ load_properties_dialog (ECategoriesDialog *parent)
 	gtk_window_set_transient_for (GTK_WINDOW (prop_dialog->the_dialog), GTK_WINDOW (parent));
 
 	prop_dialog->category_name = GTK_WIDGET (gtk_builder_get_object (prop_dialog->gui, "category-name"));
-	prop_dialog->category_icon = GTK_WIDGET (gtk_builder_get_object (prop_dialog->gui, "category-icon"));
+	table = GTK_WIDGET (gtk_builder_get_object (prop_dialog->gui, "table-category-properties"));
 
-	if (prop_dialog->category_icon) {
-		GtkFileChooser *chooser = GTK_FILE_CHOOSER (prop_dialog->category_icon);
+	if (table) {
+		GtkFileChooser *chooser;
+		GtkWidget *dialog, *button;
+		GtkWidget *image = gtk_image_new ();
 
-		if (chooser) {
-			GtkWidget *image = gtk_image_new ();
+		dialog = gtk_file_chooser_dialog_new ( _("Category Icon"),
+			NULL,
+			GTK_FILE_CHOOSER_ACTION_OPEN,
+			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+			NULL);
 
-			gtk_widget_show (image);
+		button = gtk_button_new_with_mnemonic (_("_No Image"));
+		gtk_button_set_image (GTK_BUTTON (button), gtk_image_new_from_stock (GTK_STOCK_CLOSE, GTK_ICON_SIZE_BUTTON));
+		gtk_widget_show (button);
+		gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button, GTK_RESPONSE_NO);
+		gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT);
+		button = NULL;
 
-			gtk_file_chooser_set_preview_widget (chooser, image);
-			gtk_file_chooser_set_preview_widget_active (chooser, TRUE);
+		chooser = GTK_FILE_CHOOSER (gtk_file_chooser_button_new_with_dialog (dialog));
 
-			g_signal_connect (G_OBJECT (chooser), "update-preview", (GCallback) update_preview, NULL);
-		}
+		gtk_file_chooser_set_local_only (chooser, TRUE);
+
+		g_signal_connect (dialog, "response", (GCallback) file_chooser_response, chooser);
+
+		prop_dialog->category_icon = GTK_WIDGET (chooser);
+		gtk_widget_show (prop_dialog->category_icon);
+		gtk_table_attach (GTK_TABLE (table), prop_dialog->category_icon, 1, 2, 2, 3, GTK_FILL, GTK_FILL, 0, 0);
+
+		gtk_widget_show (image);
+
+		gtk_file_chooser_set_preview_widget (chooser, image);
+		gtk_file_chooser_set_preview_widget_active (chooser, TRUE);
+
+		g_signal_connect (G_OBJECT (chooser), "update-preview", (GCallback) update_preview, NULL);
 	}
 
 	return prop_dialog;
@@ -233,7 +265,7 @@ add_comma_sep_categories (gpointer key, gpointer value, gpointer user_data)
 	GString **str = user_data;
 
 	if (strlen ((*str)->str) > 0)
-		*str = g_string_append (*str, ", ");
+		*str = g_string_append (*str, ",");
 
 	*str = g_string_append (*str, (const gchar *) key);
 }
@@ -401,9 +433,11 @@ edit_button_clicked_cb (GtkButton *button, gpointer user_data)
 
 		category_icon = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (prop_dialog->category_icon));
 
+		e_categories_set_icon_file_for (category_name, category_icon);
+
 		if (category_icon) {
 			GdkPixbuf *icon = NULL;
-			e_categories_set_icon_file_for (category_name, category_icon);
+
 			icon = gdk_pixbuf_new_from_file (category_icon, NULL);
 			if (icon) {
 				gtk_list_store_set (GTK_LIST_STORE (model), &iter, COLUMN_ICON, icon, -1);
@@ -411,7 +445,10 @@ edit_button_clicked_cb (GtkButton *button, gpointer user_data)
 			}
 
 			g_free (category_icon);
+		} else {
+			gtk_list_store_set (GTK_LIST_STORE (model), &iter, COLUMN_ICON, NULL, -1);
 		}
+
 		gtk_dialog_set_response_sensitive (GTK_DIALOG (dialog), GTK_RESPONSE_OK, TRUE);
 	}
 
@@ -716,11 +753,28 @@ const gchar *
 e_categories_dialog_get_categories (ECategoriesDialog *dialog)
 {
 	GtkEntry *entry;
+	const gchar *text;
 
 	g_return_val_if_fail (E_IS_CATEGORIES_DIALOG (dialog), NULL);
 
 	entry = GTK_ENTRY (dialog->priv->categories_entry);
 
+	text = gtk_entry_get_text (entry);
+	if (text) {
+		gint len = strlen (text), old_len = len;
+
+		while (len > 0 && (text [len -1] == ' ' || text [len - 1] == ','))
+			len--;
+
+		if (old_len != len) {
+			gchar *tmp = g_strndup (text, len);
+
+			gtk_entry_set_text (entry, tmp);
+
+			g_free (tmp);
+		}
+	}
+
 	return gtk_entry_get_text (entry);
 }
 
@@ -751,7 +805,10 @@ e_categories_dialog_set_categories (ECategoriesDialog *dialog,
 	if (arr) {
 		gint i = 0;
 		while (arr[i] != NULL) {
-			g_hash_table_insert (priv->selected_categories, g_strdup (arr[i]), g_strdup (arr[i]));
+			arr[i] = g_strstrip (arr[i]);
+
+			if (arr[i][0])
+				g_hash_table_insert (priv->selected_categories, g_strdup (arr[i]), g_strdup (arr[i]));
 			i++;
 		}
 
